@@ -12,15 +12,21 @@ static DefaultInputSource default_input_sources[] =
   { "ast_ES",   "xkb",          "es+ast" },
   { "az_AZ",    "xkb",          "az" },
   { "be_BY",    "xkb",          "by" },
+  { "bg_BG",    "xkb",          "bg+phonetic" },
   { "bn_IN",    "ibus",         "m17n:bn:inscript" },
   { "cat_ES",   "xkb",          "es+cat" },
   { "cs_CZ",    "xkb",          "cz" },
+  { "de_CH",    "xkb",          "ch" },
   { "de_DE",    "xkb",          "de" },
   { "el_CY",    "xkb",          "gr" },
   { "el_GR",    "xkb",          "gr" },
   { "en_GB",    "xkb",          "gb" },
   { "en_US",    "xkb",          "us" },
+  { "en_ZA",    "xkb",          "za" },
   { "es_ES",    "xkb",          "es" },
+  { "es_GT",    "xkb",          "latam" },
+  { "es_MX",    "xkb",          "latam" },
+  { "fr_CH",    "xkb",          "ch+fr" },
   { "fr_FR",    "xkb",          "fr+oss" },
   { "gl_ES",    "xkb",          "es" },
   { "gu_IN",    "ibus",         "m17n:gu:inscript" },
